@@ -6,7 +6,7 @@
  * 	Gonzalo Paniagua Javier
  *
  * Copyright (c) 2002 Daniel Lopez Ridruejo
- *           (c) 2002-2005 Novell, Inc.
+ *           (c) 2002-2006 Novell, Inc.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -134,6 +134,7 @@ set_auto_application (cmd_parms *cmd, void *mconfig, const char *value)
 			return apr_pstrdup (cmd->pool, "Conflicting values for MonoAutoApplication.");
 
 		sconfig->auto_app = FALSE;
+		/* TODO: Copiar de 'XXGLOBAL' a 'default' */
 	} else if (!strcasecmp (value, "enabled")) {
 		if (sconfig->auto_app_set && sconfig->auto_app != TRUE)
 			return apr_pstrdup (cmd->pool, "Conflicting values for MonoAutoApplication.");
@@ -178,7 +179,7 @@ add_xsp_server (apr_pool_t *pool, const char *alias, module_cfg *config, int is_
 	server->listen_address = NULL;
 	server->max_cpu_time = NULL;
 	server->max_memory = NULL;
-	server->debug = "False";
+	server->debug = NULL;
 	server->env_vars = NULL;
 	server->status = FORK_NONE;
 	server->is_virtual = is_virtual;
@@ -211,20 +212,43 @@ store_config_xsp (cmd_parms *cmd, void *notused, const char *first, const char *
 	offset = (unsigned long) cmd->info;
 	DEBUG_PRINT (1, "store_config %lu '%s' '%s'", offset, first, second);
 	config = ap_get_module_config (cmd->server->module_config, &mono_module);
-	if (!config->auto_app_set)
-		config->auto_app = FALSE; /* disable autoapp if there's any other application */
-
 	if (second == NULL) {
+		if (config->auto_app) {
+			idx = search_for_alias ("XXGLOBAL", config);
+			ptr = (char *) &config->servers [idx];
+			ptr += offset;
+			value = first;
+
+			/* MonoApplications/AddMonoApplications are accumulative */
+			if (offset == APR_OFFSETOF (xsp_data, applications))
+				prev_value = *((char **) ptr);
+
+			if (prev_value != NULL) {
+				new_value = apr_pstrcat (cmd->pool, prev_value, ",", value, NULL);
+			} else {
+				new_value = apr_pstrdup (cmd->pool, value);
+			}
+
+			*((char **) ptr) = new_value;
+			return NULL;
+		}
 		alias = "default";
-		if (cmd->server->is_virtual) alias = cmd->server->server_hostname;
+		if (cmd->server->is_virtual)
+			alias = cmd->server->server_hostname;
 		value = first;
 		is_default = 1;
 	} else {
+		if (!strcmp (first, "XXGLOBAL"))
+			return apr_pstrdup (cmd->pool, "XXGLOBAL is a reserved application identifier.");
 		alias = first;
 		value = second;
 		is_default = (!strcmp (alias, "default"));
 	}
 
+	/* Disable autoapp if there's any other application. MonoDebug is excluded. */
+	if (!config->auto_app_set)
+		config->auto_app = FALSE;
+
 	idx = search_for_alias (alias, config);
 	if (idx == -1)
 		idx = add_xsp_server (cmd->pool, alias, config, is_default, cmd->server->is_virtual);
@@ -1117,7 +1141,7 @@ fork_mod_mono_server (apr_pool_t *pool, xsp_data *config)
 	memset (argv, 0, sizeof (char *) * MAXARGS);
 	argi = 0;
 	argv [argi++] = config->executable_path;
-	if (!strcasecmp (config->debug, "True"))
+	if (config->debug && !strcasecmp (config->debug, "True"))
 		argv [argi++] = "--debug";
 
 	argv [argi++] = config->server_path;
@@ -1395,6 +1419,7 @@ mono_execute_request (request_rec *r, char auto_app)
 		idx = search_for_alias (NULL, config);
 
 	DEBUG_PRINT (2, "idx = %d", idx);
+
 	if (idx < 0) {
 		DEBUG_PRINT (2, "Alias not found. Checking for auto-applications.");
 		if (config->auto_app)
